#ifndef VIEW_H
#define VIEW_H

#include <qevent.h>
#include <qptrlist.h>
#include <qpushbutton.h>
#include <qdict.h>

#include <klistview.h>
#include <kmainwindow.h>
#include <kio/global.h>

#include <noatun/downloader.h>
#include <noatun/stdaction.h>


#define FPL FlexPlaylist::FPL()

// Noatun in KDE 3.1.x is V2.2.1
// Noatun in KDE 3.2 (CVS) is V2.3.2 currently and will be release as V2.4.0
#define NOATUN_2_3 (NOATUN_MAJOR == 2 && NOATUN_MINOR >= 3)

#define TITLE_COL 0
#define TIME_COL 1


class Finder;
class View;
class FileJumpDialog;

class KFileDialog;
class KToggleAction;
class KToolBar;
class KPopupMenu;

namespace KIO { class ListJob; }

class TrayButton : public QPushButton
{
	Q_OBJECT
	public:
		TrayButton(QWidget*, const char*);
		virtual ~TrayButton() {}

	protected:
		void drawButton(QPainter *);
};

class SafeListViewItem
	: public KListViewItem
	, public PlaylistItemData
	, public DownloadItem
{
public:
	SafeListViewItem(QListView *parent, QListViewItem *after, const KURL &text);
	SafeListViewItem(QListView *parent, QListViewItem *after, const QMap<QString,QString> &properties);
	virtual ~SafeListViewItem();

	virtual QString property(const QString &, const QString & = 0) const;
	virtual void setProperty(const QString &, const QString &);
	virtual void clearProperty(const QString &);
	virtual QStringList properties() const;
	virtual bool isProperty(const QString &) const;

	virtual void remove();

	virtual QString file() const;

	int compare(QListViewItem * i, int col, bool ascending) const;

	virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment);

	void updatePixmap();

protected:
	virtual void downloaded(int percent);
	virtual void downloadTimeout();
	virtual void downloadFinished();
	virtual void modified();
	//virtual int width(const QFontMetrics &fm, const QListView *lv, int c) const;

private:
	struct Property
	{
		QString key;
		QString value;
	};
	QValueList<Property> mProperties;
	bool removed;
};

// --------------------------------------------------------------------------------------

class FPListView : public KListView
{
	Q_OBJECT
	friend class View;

	public:
		FPListView(View *parent, const char *name = "FPListView");
		virtual ~FPListView();

		QListViewItem *loadPlaylist(const KURL&);
		QListViewItem *addPlaylist(const KURL&, QListViewItem * =0);
		QListViewItem *addFile(const KURL&, bool play=false, QListViewItem * =0);
		void addDirectoryRecursive(const KURL &dir, QListViewItem * =0);

	public slots:
		virtual void clear();
		/*
		 * moves the selected item(s) to immediately after the currently
		 * playing item
		 */
		void moveSelectionAfterCurrent();
		void removeFromDisk();
		void deleteSelected();
		void purge(); // removes all playlistitems pointing to non-existant local files

	signals:
		void modified(void);
		void deleteCurrentItem();

	protected:
		virtual bool acceptDrag(QDropEvent *event) const;
		virtual void keyPressEvent(QKeyEvent *e);
		virtual void viewportResizeEvent(QResizeEvent *e);

	protected slots:
		virtual void dropEvent(QDropEvent *event, QListViewItem *after);
		void move();
		void slotUpdateColumnWidths();

	protected:
		QListViewItem *recursiveAddAfter;

	protected slots:
		// used when adding directories via KIO::listRecursive
		void slotResult(KIO::Job *job);
		void slotEntries(KIO::Job *job, const KIO::UDSEntryList &entries);
		void slotRedirection(KIO::Job *, const KURL & url);
		void slotDropMenuActivated(int);

	protected:
		void addNextPendingDirectory();
		KURL::List pendingAddDirectories;
		KIO::ListJob *listJob;
		KURL currentJobURL;
		/*
		 * the menu shown on dropping something on the list
		 * and some temp vars used for after a selection in the
		 * popup was made
		 */
		KPopupMenu *mDropMenu;
		KURL::List dropItemList;
		QListViewItem *dropAfterItem;
};

// --------------------------------------------------------------------------------------

class View : public KMainWindow
{
	 Q_OBJECT
	 public:
		View(const char *name=0L);
		void init(); // load the SM playlist
		virtual ~View();

		FPListView *listView() const { return list; }
		QListViewItem *addFile(const KURL &u, bool play=false)
			 { return list->addFile(u, play, list->lastItem()); }

	 public slots:
		void addFiles();
		void addDirectory();
		void save();
		void saveAs();
		void open();
		/*
		 * This method is like openNew() but it
		 * doesn't prompt for a filename
		 */
		void fileNew();
		void setSorting(bool on, int column = 0);
		void setNoSorting();
		void headerClicked(int column);
		void find();
		void findIt(Finder *);
		void slotSelectionChanged();
		void updateStatusBar();

		/** shows a context menu for item i at point p */
		void showItemContextMenu(KListView* l, QListViewItem* i, const QPoint& p);
		/**
		* This method prompts user to save if the playlist is modified, and
		* handles saving if they say yes.
		* @return true if user selected yes or no, false if they selected cancel
		*/
		bool saveModified();
		/** opens a properties dialog for the currently selected item(s) */
		void showItemProperties();
		/** plays current item */
		void playCurrent();

	private slots:
		void setModified();
		void saveState();
		void slotLoadWindowState();
		void slotSaveWindowState();
		void slotConfKeys();
		void slotOpenPrefs();

		void slotJumpTo();
		void slotGoToCurrent();

		void configureToolBars();
		void newToolBarConfig();
		void slotReadConfig();
		void showMenubar();
		void showStatusbar();
		void playing();
		void notplaying();

		void slotDoJump(const PlaylistItem&);

	protected:
		void setupActions();
		bool saveToURL(const KURL &);
		void setModified(bool);
		virtual void closeEvent(QCloseEvent*e);
		virtual void showEvent(QShowEvent *);
		virtual void hideEvent(QHideEvent *);

	signals:
		void hidden();
		void shown();

	private:
		FPListView *list;
		Finder *mFinder;

		KAction *mTrash;
		KAction *mDelete;
		// moves the selected item(s) directly after the currently playing item
		KAction *mPlaySelectedNext;
		// plays selected item
		KAction *mPlayCurrent;
		// shows a properties dialog for the selected item(s)
		KAction *mItemProperties;
		// show/hide toolbar, menubar and statusbar
		KToggleAction *menubarAction, *statusbarAction;

		TrayButton *mPlayPauseButton, *mStopButton;
		FileJumpDialog *mFileJumpDialog;
		QHBox *mStatusBarWidget;

		KURL mPlaylistFile;
		bool modified;
		unsigned long fullLength;
};
#endif
// vim: set noet ts=4 sts=4 sw=4:
